%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% MP3 Bit Rate Quality Detection through Frequency Spectrum Analysis
%
%    This code performs SVM classification of frequency spectrum data in order to predict the bit rate of an mp3 file
%    Copyright (C) 2009  Brian D'Alessandro, Ph.D.
%    http://www.briandalessandro.com/
% 
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU Affero General Public License as
%    published by the Free Software Foundation, either version 3 of the
%    License, or (at your option) any later version.
% 
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU Affero General Public License for more details.
% 
%    You should have received a copy of the GNU Affero General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%    For further information, please refer to the paper:
%
%    Brian D'Alessandro, Yun Q. Shi, "MP3 Bit Rate Quality Detection through Frequency Spectrum Analysis," 
%    in Proceedings of the 11th ACM Workshop on Multimedia and Security (MM&Sec 09), Princeton, NJ, September 7-8, 2009, pp. 57-62. 
%    DOI: 10.1145/1597817.1597828
% 
%    PDF available at: http://www.briandalessandro.com/about/cv.php
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function allFeatures = GenerateMp3Features(infolder)

    %Find mp3 files in the target directory
    list = dir(fullfile(infolder, '*.mp3'));
    nfiles = length(list);
    
    %Initialize feature array for all songs, 142xN where 142 is the number of features & N is the number of mp3 files
    allFeatures = zeros(142, nfiles);
    
    %Loop over all mp3 files in the directory
    for m=1:nfiles
        %clear variables
        clear y F Fs P S T db db_new spec ss status curdir;
        
        %Execute mp3sDecoder to convert mp3 into wav
        cmd=['mp3towav.exe -if "', fullfile(infolder, list(m).name), '" -of ', 'tmp.wav'];
        [status currdir] = system(cmd);
        
        %Have matlab read wav data
        [y, Fs, nbits] = wavread('tmp.wav');
        delete('tmp.wav');
        
        %Find spectrogram of the audio file
        [S,F,T,P] = spectrogram(y(:,1), blackmanharris(1024), 0, 1024, Fs);
        
        %Find db of the frequency range of interest... (372:513) corresponds to frequencies above 16kHz
        db = 10*log10(abs(P(372:513,:).'));
        
        %get rid of infinite entries
        ninf = sum(isinf(db'));
        db_new = db(ninf==0,:);
        
        %Average frequency data across the entire length of the song
        spec = mean( db_new );

        %Add current mp3's features to the array of all features for all songs 
        allFeatures(1:142, m)=spec;
    end
    
    
    