/*  svm.h

    Head file of SVM.

    Developed from LIBSVM 2.33.

    Modified by Yi Zhao, OSU-EE, 01/20/2002 (version 2.00).
    Modified by Yi Zhao, OSU-EE, 12/31/2001 (version 1.10).
    Modified by Yi Zhao, OSU-EE, 12/22/2001 (version 1.00).

    Modifiedy by Junshui Ma, 02/12/2002 to fit in the 
	the OSU SVM framework.
*/

#ifndef __LIBSVM_H_
#define __LIBSVM_H_

#ifdef __cplusplus
extern "C" {
#endif

// svm_type
//
enum {
	C_SVC,
	NU_SVC,
	ONE_CLASS,
	EPSILON_SVR,
	NU_SVR
};

// kernel_type
//
enum {
	LINEAR,
	POLY,
	RBF,
	SIGMOID
};

// svm_node
//
typedef struct _svm_node {
	int index;
	double value;
} svm_node;

// svm_problem
//
typedef struct _svm_problem {
	int l;
	double *y;
	svm_node **x;
} svm_problem;

// svm_parameter
//
typedef struct _svm_parameter {
	int svm_type;		// SVM type
	int kernel_type;	// kernel type
	double degree;		// for poly
	double gamma;		// for poly/rbf/sigmoid
	double coef0;		// for poly/sigmoid
	// these are for training only
	double cache_size;	// in MB
	double eps;		// stopping criteria
	double C;		// for C_SVC, EPSILON_SVR and NU_SVR
	int nr_weight;	// for C_SVC
	int *weight_label;	// for C_SVC
	double* weight;		// for C_SVC
	double nu;	// for NU_SVC, ONE_CLASS, and NU_SVR
	double p;	// for EPSILON_SVR
	int shrinking;	// use the shrinking heuristics
} svm_parameter;

// svm_model
//
typedef struct _svm_model {
	svm_parameter param;	// kernel parameter
	int nr_class;	// number of classes, = 2 in regression/one class svm
	int l;			// total #SV
	svm_node **SV;		// SVs (SV[l])
	double **sv_coef;	// coefficients for SVs in decision functions (sv_coef[n-1][l])
	double *rho;		// constants in decision functions (rho[n*(n-1)/2])
	// for classification only
	int *label;		// label of each class (label[n])
	int *nSV;		// number of SVs for each class (nSV[n])
					//   nSV[0] + nSV[1] + ... + nSV[n-1] = l
	// XXX
	int free_sv;	// 1 if svm_model is created by svm_load_model
					// 0 if svm_model is created by svm_train
} svm_model;

// decision_function
//
typedef struct _decision_function {
	double *alpha;
	double rho;	
} decision_function;

// prediction_output
typedef struct _prediction_output {
	double label;
	double value;	
} prediction_output;
 


/* Function Prototype. */
svm_model * svm_train		(const svm_problem *prob,
		const svm_parameter *param);
prediction_output svm_predict	(const svm_model *model, const svm_node *x);
int svm_save_model	(const char *model_file_name,
		const svm_model *model);
svm_model * svm_load_model	(const char * model_file_name);
void svm_destroy_model	(svm_model *model);

#ifdef __cplusplus
}
#endif

#endif	/* __LIBSVM_H_ */
